import java.util.*;

/**
 * Person identity using the Bloch
 * Builder from 'Effective Java' to construct complex state.
 *
 * Original source code is from the Net4Care PHMR Builder library:
 * https://bitbucket.org/4s/net4care-phmr-builder
 *
 
   This source code is from the book 
     "Flexible, Reliable Software, 2nd Edition:
       Using Patterns and Agile Development"
     First Edition published by CRC Press, 2010.
     Second Edition, available for download
       by permission.

   Author: 
     Henrik Bærbak Christensen 
     Department of Computer Science
     Aarhus University
   
   Please visit http://www.baerbak.com/ for further information.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
 
       http://www.apache.org/licenses/LICENSE-2.0
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

 */
public final class PersonIdentity {

  private String ssn;
  private String title;
  private String[] givenNames;
  private String familyName;
  private Gender gender;
  private Date birthTime;

  private String[] phoneNumbers;
  
  /** "Effective Java" builder for constructing person identities.
   *
   * @author Henrik Baerbak Christensen, Aarhus University
   *
   */
  public static class PersonBuilder {
    // Required parameters
    private String familyName;
    private String[] givenNames;

    // Optional parameters
    private String personId = null;
    private Gender gender = Gender.Undifferentiated;
    private Date birthTime = null;
    private String[] phoneNumberArray = null;
    private String title = null;
    
    // temporaries
    private int yearAllDigits = 0;
    private int dayInMonth = 0;
    private int month = 0;
    private List<String> givenNamesTemporary;
    private List<String> phoneNumberList;

    public PersonBuilder() {
      givenNamesTemporary = new ArrayList<String>();
      phoneNumberList = new ArrayList<String>();
    }

    public PersonBuilder(String familyName) {
      givenNamesTemporary = new ArrayList<String>();
      phoneNumberList = new ArrayList<String>();

      this.familyName = familyName;
    }

    public PersonBuilder setFamilyName(String familyName) {
      this.familyName = familyName;
      return this;
    }

    public PersonBuilder addGivenName(String name) {
      givenNamesTemporary.add(name);
      return this;
    }
    public PersonBuilder setPersonID(String personId) {
      this.personId = personId;
      return this;
    }
    public PersonBuilder setGender(Gender gender) {
      this.gender = gender;
      return this;
    }
    public PersonBuilder setTitle(String title) {
      this.title = title;
      return this;
    }
    public PersonBuilder setBirthTime(int yearAllDigits, 
                                      int monthUsingCalendarEnums, 
                                      int dayInMonth) {
      this.yearAllDigits = yearAllDigits;
      this.month = monthUsingCalendarEnums;
      this.dayInMonth = dayInMonth;
      return this;
    }

    public PersonBuilder addPhoneNumber(String phoneNumber) {
      phoneNumberList.add(phoneNumber);
      return this;
    }

    public PersonIdentity build() {
      givenNames = new String[givenNamesTemporary.size()];
      givenNamesTemporary.toArray(givenNames);

      if (phoneNumberList.size() > 0) {
        phoneNumberArray = new String[ phoneNumberList.size()];
        phoneNumberList.toArray(phoneNumberArray);
      }

      if (yearAllDigits != 0) {
        Calendar utcCalendar = 
          Calendar.getInstance(TimeZone.getTimeZone("UTC"));
        utcCalendar.set(yearAllDigits, month, dayInMonth, 0, 0, 0);
        birthTime = utcCalendar.getTime();
      }
      // Constraint based setting of title
      if (title == null) {
        if (gender == Gender.Male) {
          title = "Mr.";
        } else {
          title = "Miss";
        }
      }
      PersonIdentity pi = new PersonIdentity(this);
      return pi;
    }
  }

  private PersonIdentity(PersonBuilder builder) {
    familyName = builder.familyName;
    givenNames = builder.givenNames;
    ssn = builder.personId;
    gender = builder.gender;
    birthTime = builder.birthTime;
    phoneNumbers = builder.phoneNumberArray;
    title = builder.title;
  }

  /** Get the social security number.
   * @return The social security number.
   */
  public String getSSN() {
    return ssn;
  }

  /** Get person title.
   * @return title string.
   */
  public String getTitle() {
    return title;
  }

  /** Get list of given names.
   * @return list of given names.
   */
  public String[] getGivenNames() {
    return givenNames;
  }

  /** Get family name.
   * @return family name string.
   */
  public String getFamilyName() {
    return familyName;
  }

  /** Get person gender.
   * @return gender type.
   */
  public Gender getGender() {
    return gender;
  }

  /** Get person birth time.
   * @return birth time date.
   */
  public Date getBirthTime() {
    return birthTime;
  }

  /** Get person phone numbers.
   * @return phone number array.
   */
  public String[] getPhoneNumbers() {
    if (phoneNumbers == null) {
      phoneNumbers = new String[]{}; // Empty array
    }
    return phoneNumbers;
  }

  public String toString() {
    String result = "PersonIdentity:\n";

    result += " Title         : " + title + "\n";
    result += " Family name   : " + getFamilyName() + "\n";
    result += " Given name(s) : ";
    for (String firstname: getGivenNames()) {
      result += firstname + " / ";
    }
    result += "\n";

    result += " Gender        : " + getGender() + "\n";
    
    result += " SSN/ID        : " + getSSN() + "\n";
    
    result += " Birth         : " + getBirthTime() + "\n";
    
    if (phoneNumbers != null) {
      result += " Phone         : [";
      for (String tc : getPhoneNumbers()) {
        result += tc + ",";
      }
      result += "]\n";
    }
    
    result += "---";

    return result;
  }

}
