import java.util.*;

/** Demonstration of the Composite Pattern.
 
   This source code is from the book 
     "Flexible, Reliable Software, 2nd Edition:
       Using Patterns and Agile Development"
     First Edition published by CRC Press, 2010.
     Second Edition, available for download
       by permission.

   Author: 
     Henrik Bærbak Christensen 
     Department of Computer Science
     Aarhus University
   
   Please visit http://www.baerbak.com/ for further information.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
 
       http://www.apache.org/licenses/LICENSE-2.0
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/
public class CompositeDemo {
  public static void main(String[] args) {
    System.out.println( "======= Demonstration of Composite =======");
    // make the root of my hard disk
    Component root = new Folder();
    // create some folder in a hierarchy
    // root
    //   --- a
    //       --- b
    //   --- c
    Component folder_a = new Folder();
    root.addComponent(folder_a);

    Component folder_b = new Folder();
    folder_a.addComponent(folder_b);

    Component folder_c = new Folder();
    root.addComponent(folder_c);

    // add some 'files' to the a folder
    Component 
      file_x = new File(1000),
      file_y = new File(3000),
      file_z = new File(5000);
      
    folder_a.addComponent(file_x);
    folder_a.addComponent(file_y);
    folder_a.addComponent(file_z);

    // add some 'files' to the b folder
    Component 
      file_1 = new File(30),
      file_2 = new File(20),
      file_3 = new File(10);
      
    folder_b.addComponent(file_1);
    folder_b.addComponent(file_2);
    folder_b.addComponent(file_3);
 
    // leaf operation
    System.out.println( "File 1 has size "+file_1.size()+ 
                        " (Correct: 30)" );
    // composite operation
    System.out.println( "Folder b has size "+folder_b.size()+ 
                        " (Correct: 60)" );
    System.out.println( "Folder root has size "+root.size()+ 
                        " (Correct: 9060)" );
  }
}

/** Define the Component interface 
 * (partial for a folder hierarchy) */
interface Component {
  public void addComponent(Component child);
  public int size();
}                         

/** Define a (partial) file abstraction. */
class File implements Component {
  private int size;
  public File(int size) { this.size = size; }
  public void addComponent(Component child) {
    // no-operation, does not make sense.
  }
  public int size() { return size; }
}
  
/** Define a (partial) folder abstraction */
class Folder implements Component {
  private List<Component> components = new ArrayList<Component>();
  public void addComponent(Component child) {
    components.add(child);
  }
  public int size() { 
    int size = 0;
    for ( Component c: components ) {
      size += c.size();
    }
    return size;
  }
}
