package paystation.domain;

import org.junit.jupiter.api.Test;

import static org.hamcrest.CoreMatchers.*;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.closeTo;
import static org.junit.jupiter.api.Assertions.assertEquals;

/** Testcases for the Pay Station system.
 
   This source code is from the book 
     "Flexible, Reliable Software:
       Using Patterns and Agile Development"
     published 2010 by CRC Press.
   Author: 
     Henrik B Christensen 
     Computer Science Department
     Aarhus University
   
   This source code is provided WITHOUT ANY WARRANTY either 
   expressed or implied. You may study, use, modify, and 
   distribute it for non-commercial purposes. For any 
   commercial use, see http://www.baerbak.com/
*/
public class TestPayStation {

  private PayStation ps;

  /** Test acceptance of all legal coins using the One2One rate strategy */
  @Test
  public void shouldAcceptLegalCoins() throws IllegalCoinException {
    ps = new PayStationImpl(privatePayStation -> privatePayStation.getInsertedSoFar());
    ps.addPayment( 5 );
    ps.addPayment( 10 );
    ps.addPayment( 25 );
    assertThat(ps.readDisplay(), is(5+10+25));
  }

  /**
   * Buy should return a valid receipt of the
   * proper amount of parking time
   */
  @Test
  public void shouldReturnCorrectReceiptWhenBuy()
          throws IllegalCoinException {
    ps = new PayStationImpl(privatePayStation -> privatePayStation.getInsertedSoFar());
    ps.addPayment(5);
    ps.addPayment(10);
    ps.addPayment(25);
    Receipt receipt;
    receipt = ps.buy();
    assertThat(receipt.value(), is(5+10+25));
  }

  @Test
  public void shouldVerifyOmegaTown() throws IllegalCoinException {
    RateStrategy rs = new OmegaRateStrategy();
    // Given a specification: when 1 $ has been
    // entered an immediate discount of 25c is provided
    // by instantly increasing the 'insertedSoFar' from
    // 1$ to 1.25$. Each cent gives 2 minute parking
    ps = new PayStationImpl(rs);
    // When entering 75 cent
    ps.addPayment(25);
    ps.addPayment(25);
    ps.addPayment(25);
    // Then 150 minutes is in the display
    assertThat(ps.readDisplay(), is(2*75));
    // When entering further 20 cent
    ps.addPayment(10); ps.addPayment(10);
    // Then 190 minutes is in the display
    assertThat(ps.readDisplay(), is(2*95));

    // WHEN we enter 5 cent more, we pass the 1$ limit and
    ps.addPayment(5);
    // THEN the insertedSoFar is suddenly 125 cent
    assertThat(ps.readDisplay(), is(2*125));
  }
}
