package paystation.domain;

import org.junit.jupiter.api.*;
import static org.junit.jupiter.api.Assertions.*;

import static org.hamcrest.MatcherAssert.assertThat; 
import static org.hamcrest.Matchers.*;

import java.io.*;

/** Integration testing of the configurations of the pay station.
 
   This source code is from the book 
     "Flexible, Reliable Software, 2nd Edition:
       Using Patterns and Agile Development"
     First Edition published by CRC Press, 2010.
     Second Edition, available for download
       by permission.

   Author: 
     Henrik Bærbak Christensen 
     Department of Computer Science
     Aarhus University
   
   Please visit http://www.baerbak.com/ for further information.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
 
       http://www.apache.org/licenses/LICENSE-2.0
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/
public class TestIntegration {
  private PayStation ps;
  
  /**
   * Integration testing for the linear rate configuration
   */
  @Test 
  public void shouldConfigureAlphaTownCorrectly() 
          throws IllegalCoinException {
    // Given a AlphaTown paystation / linear rate
    ps = new PayStationImpl(new AlphaTownFactory());
    // When adding 2$
    addOneDollar(); addOneDollar();

    // Then the display reads 80 minutes; but the
    // time based output strategy makes that hard to
    // test without introducing test doubles which
    // is too costly to introduce compared to benefit
    boolean isInRange = ps.readDisplay() >= 0 && 
      ps.readDisplay() < 2359;
    assertThat(isInRange, is(true));

    // Then the receipt is 5 lines long = standard receipt
    assertThat(buyReturnsReceiptOfLength(ps), is(5));
  }

  /**
   * Integration testing for the progressive rate configuration
   */
  @Test 
  public void shouldConfigureBetaTownCorrectly() 
          throws IllegalCoinException {
    // Given a BetaTown pay station / progressive rate
    ps = new PayStationImpl(new BetaTownFactory());
    // When adding 2$
    addOneDollar(); addOneDollar();

    // Then the display reads 75 minutes
    assertThat(ps.readDisplay(), is(75));

    // Then the receipt is 6 lines long = bar code receipt
    assertThat(buyReturnsReceiptOfLength(ps), is(6));
  }

  /**
   * Integration testing for gamma town. This integration test will
   * need to switch between either weekdays or weekend to test
   * that indeed the Gammatown works correctly. The test
   * thus becomes pretty complex, and 'non-evident' and
   * arguably more complex than the production code it tests.
   * Never-the-less I have kept it for the sake of demonstration.
   */
  @Test 
  public void shouldConfigureGammaTownCorrectly() 
    throws IllegalCoinException {
    // Given a paystation to be the Gamma town pay station, BUT
    // using the stub to set for normal working days
    ps = new PayStationImpl(new GammaTownFactory() {
      public RateStrategy createRateStrategy() {
        return new AlternatingRateStrategy( new LinearRateStrategy(),
                                            new ProgressiveRateStrategy(),
                                            new FixedDecisionStrategy(false) );
      }});
    // When adding 2$
    addOneDollar(); addOneDollar();
    
    // Then the display reads 80 minutes
    assertThat(ps.readDisplay(), is(80));
    
    // Then the receipt is 5 lines long = standard receipt
    assertThat(buyReturnsReceiptOfLength(ps), is(5));

    // Given a Gamma town paystation during weekend
    ps = new PayStationImpl(new GammaTownFactory() {
      public RateStrategy createRateStrategy() {
        return new AlternatingRateStrategy( new LinearRateStrategy(),
                                            new ProgressiveRateStrategy(),
                                            new FixedDecisionStrategy(true) );
      }});
    // When adding 2$
    addOneDollar(); addOneDollar();
    
    // Then the display reads 75 minutes
    assertThat(ps.readDisplay(), is(75));
  }


  private void addOneDollar() throws IllegalCoinException {
    ps.addPayment(25); ps.addPayment(25); 
    ps.addPayment(25); ps.addPayment(25); 
  }

  private int buyReturnsReceiptOfLength(PayStation ps) {
    Receipt receipt = ps.buy();
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    PrintStream stream = new PrintStream(baos);
    receipt.print(stream);
    String output = baos.toString();
    String[] lines = output.split("\n");
    return lines.length;
  }
}
