package paystation.domain;

import org.junit.jupiter.api.*;
import static org.junit.jupiter.api.Assertions.*;

import static org.hamcrest.MatcherAssert.assertThat; 
import static org.hamcrest.Matchers.*;

/** Test cases for the Pay Station system.
 
   This source code is from the book 
     "Flexible, Reliable Software, 2nd Edition:
       Using Patterns and Agile Development"
     First Edition published by CRC Press, 2010.
     Second Edition, available for download
       by permission.

   Author: 
     Henrik Bærbak Christensen 
     Department of Computer Science
     Aarhus University
   
   Please visit http://www.baerbak.com/ for further information.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
 
       http://www.apache.org/licenses/LICENSE-2.0
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/
public class TestPayStation {
  PayStation ps;
  /** Fixture for pay station testing. */
  @BeforeEach
  public void setUp() {
    // Given a PayStation whose rate strategy is that
    // one cent buys one minute parking time
    ps = new PayStationImpl(coinValue -> coinValue); 
  }

  /** Test acceptance of all legal coins */
  @Test
  public void shouldAcceptLegalCoins() throws IllegalCoinException {
    // Given a paystation
    // When I enter 5, 10, and 25 cents
    ps.addPayment(5);
    ps.addPayment(10);
    ps.addPayment(25);
    // Then the display should read 40
    assertThat(ps.readDisplay(), is(5+10+25)); 
  }

  /** 
   * Verify that illegal coin values are rejected.
  */
  @Test
  public void shouldRejectIllegalCoin() throws IllegalCoinException {
    assertThrows(IllegalCoinException.class, () -> {
        ps.addPayment(17);
      });
  }

  /**
   * Buy should return a valid receipt of the 
   * proper amount of parking time
  */
  @Test 
  public void shouldReturnCorrectReceiptWhenBuy() 
          throws IllegalCoinException {
    // Given a paystation,
    // When I enter 40 cents
    ps.addPayment(5);
    ps.addPayment(10);
    ps.addPayment(25);

    Receipt receipt;
    receipt = ps.buy();
    // Then the receipt is not null
    assertThat(receipt, is(notNullValue()));
    // Then the receipt reads 40 minutes
    assertThat(receipt.value(), is(5+10+25));
  }
 
  /**
   * Receipts must be able to store parking time values
   */
  @Test 
  public void shouldStoreTimeInReceipt() {
    // Given a receipt of 30 minutes
    Receipt receipt = new ReceiptImpl(30);
    // Then I can read this value correctly
    assertThat(receipt.value(), is(30));
  }

  /**
   * Buy for 100 cents and verify the receipt
  */
  @Test 
  public void shouldReturnReceiptWhenBuy100c() 
    throws IllegalCoinException {
    // Given a paystation
    // When I enter 100 cents
    ps.addPayment(10); ps.addPayment(10); ps.addPayment(10);
    ps.addPayment(10); ps.addPayment(10);
    ps.addPayment(25); ps.addPayment(25);

    Receipt receipt = ps.buy();
    // Then the receipt's value is 100 minutes
    assertThat(receipt.value(), is(5*10+2*25));
  }

  /**
   * Verify that the pay station is cleared after a buy scenario
  */
  @Test 
  public void shouldClearAfterBuy() 
          throws IllegalCoinException {
    // Given a paystation
    // When I enter 25 cents and press buy
    ps.addPayment(25);
    ps.buy(); // I do not care about the result
    // Then the display reads 0
    assertThat(ps.readDisplay(), is(0));
    
    // When I initiate the next buy transaction
    ps.addPayment(10); ps.addPayment(25);
    assertThat(ps.readDisplay(), is(10+25));
    // Then the receipt is valid
    Receipt r = ps.buy();
    assertThat(r.value(), is(10+25));
    // and Then the display is cleared
    assertThat(ps.readDisplay(), is(0) );
  }

  /**
   * Verify that cancel clears the pay station
   */
  @Test 
  public void shouldClearAfterCancel() 
          throws IllegalCoinException {
    // Given a paystation
    // When I add 10 cents and then cancels
    ps.addPayment(10);
    ps.cancel();
    // Then the display reads 0
    assertThat(ps.readDisplay(), is(0));
    // When I next enter 25
    ps.addPayment(25);
    // Then display shows the proper value
    assertThat(ps.readDisplay(), is(25));
  }
}
