import org.junit.jupiter.api.*;
import static org.junit.jupiter.api.Assertions.*;

import static org.hamcrest.MatcherAssert.assertThat; 
import static org.hamcrest.Matchers.*;

/** Test cases for the Pay Station system.
 
   This source code is from the book 
     "Flexible, Reliable Software, 2nd Edition:
       Using Patterns and Agile Development"
     First Edition published by CRC Press, 2010.
     Second Edition, available for download
       by permission.

   Author: 
     Henrik Bærbak Christensen 
     Department of Computer Science
     Aarhus University
   
   Please visit http://www.baerbak.com/ for further information.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
 
       http://www.apache.org/licenses/LICENSE-2.0
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/
public class TestPayStation {
  PayStation ps;
  /** Fixture for pay station testing. */
  @BeforeEach
  public void setUp() {
    ps = new PayStationImpl();
  }

  /**
   * Entering 5 cents should make the display report 2 minutes 
   * parking time.
  */
  @Test
  public void shouldDisplay2MinFor5Cents() throws IllegalCoinException {
    ps.addPayment(5);
    assertThat(ps.readDisplay(), is(5 / 5 * 2)); 
  }

  /**
   * Entering 25 cents should make the display report 10 minutes
   * parking time.
  */
  @Test
  public void shouldDisplay10MinFor25Cents() throws IllegalCoinException {
    ps.addPayment(25);
    assertThat(ps.readDisplay(), is(25 / 5 * 2));
    // 25 cent in 5 cent coins each giving 2 minutes parking
  }

  /** 
   * Verify that illegal coin values are rejected.
  */
  @Test
  public void shouldRejectIllegalCoin() throws IllegalCoinException {
    assertThrows(IllegalCoinException.class, () -> {
        ps.addPayment(17);
      });
  }

  /**
   * Entering 10 and 25 cents should be valid and 
   * return 14 minutes parking 
  */
  @Test
  public void shouldDisplay14MinFor10And25Cents()
          throws IllegalCoinException {
    ps.addPayment(10);
    ps.addPayment(25);
    assertThat(ps.readDisplay(), is((10+25) / 5 * 2));
  }

  /**
   * Buy should return a valid receipt of the 
   * proper amount of parking time
  */
  @Test 
  public void shouldReturnCorrectReceiptWhenBuy() 
          throws IllegalCoinException {
    ps.addPayment(5);
    ps.addPayment(10);
    ps.addPayment(25);
    Receipt receipt;
    receipt = ps.buy();
    assertThat(receipt, is(notNullValue()));
    assertThat(receipt.value(), is((5+10+25) / 5 * 2));
  }

  /**
   * Receipts must be able to store parking time values
  */
  @Test 
  public void shouldStoreTimeInReceipt() {
    Receipt receipt = new ReceiptImpl(30);
    assertThat(receipt.value(), is(30));
  }

  /**
   * Buy for 100 cents and verify the receipt
  */
  @Test 
  public void shouldReturnReceiptWhenBuy100c() 
    throws IllegalCoinException {
    ps.addPayment(10); ps.addPayment(10); ps.addPayment(10);
    ps.addPayment(10); ps.addPayment(10);
    ps.addPayment(25); ps.addPayment(25);

    Receipt receipt = ps.buy();
    assertThat(receipt.value(), is((5*10+2*25) / 5 * 2));
  }

  /**
   * Verify that the pay station is cleared after a buy scenario
  */
  @Test 
  public void shouldClearAfterBuy() 
          throws IllegalCoinException {
    ps.addPayment(25);
    ps.buy(); // I do not care about the result
    // verify that the display reads 0
    assertThat(ps.readDisplay(), is(0));
    // verify that a following buy scenario behaves properly
    ps.addPayment(10); ps.addPayment(25);
    assertThat(ps.readDisplay(), is((10+25) / 5 * 2));
    Receipt r = ps.buy();
    assertThat(r.value(), is((10+25) / 5 * 2));
    assertThat(ps.readDisplay(), is(0) );
  }
}
