package breakthrough.domain;

/**
 * Facade for a Breakthrough game. The chess board is organized in 8 rows of 8
 * columns, numbered 0..7. Black has pawns on row 0 and 1 while white has
 * pawns on row 6 and 7.
 * <pre>
   0 1 2 3 4 5 6 7
   ----------------
7: W W W W W W W W
6: W W W W W W W W
5: - - - - - - - -
4: - - - - - - - -
3: - - - - - - - -
2: - - - - - - - -
1: B B B B B B B B
0: B B B B B B B B
   ----------------
   0 1 2 3 4 5 6 7
</pre>
 * 
 * 
 
   This source code is from the book 
     "Flexible, Reliable Software, 2nd Edition:
       Using Patterns and Agile Development"
     First Edition published by CRC Press, 2010.
     Second Edition, available for download
       by permission.

   Author: 
     Henrik Bærbak Christensen 
     Department of Computer Science
     Aarhus University
   
   Please visit http://www.baerbak.com/ for further information.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
 
       http://www.apache.org/licenses/LICENSE-2.0
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

 
 */
public interface Breakthrough {

  /**
   * Return the type of piece on a given position on the chess board.
   *
   * @return the type of piece on the location.
   */
  public Color getPieceAt(Position p);

  /**
   * Return the player that is in turn, i.e. allowed to move.
   *
   * @return the player that may move a piece next
   */
  public Color getPlayerInTurn();

  /**
   * Return the winner of the game.
   *
   * @return the winner of the game or null in case no winner has been found
   *         yet.
   */
  public Color getWinner();

  /**
   * Validate and potentially make a move. A move is invalid if you try to move
   * your opponent's pieces or the move does not follow the rules, see the
   * exercise specification. PRECONDITION: the (row,column) coordinates are
   * valid positions, that is, all between (0..7).
   *
   * @return true if the move is valid, false otherwise
   */
  public boolean move(Move move);

}
