package breakthrough.domain;

/**
 * A 'surrogate' implementation for a Breakthrough game, the rules are not
 * really obeyed, but it has enough behavior to test most aspects of a Broker
 * implementation of the game - like invalid moves, getting contents of the
 * board, etc. Winner condition is not implemented for obvious reasons.
 *
 * It is kind of a test double, but does not match any of Meszaros' categories,
 * thus it is named a surrogate.

   This source code is from the book 
     "Flexible, Reliable Software, 2nd Edition:
       Using Patterns and Agile Development"
     First Edition published by CRC Press, 2010.
     Second Edition, available for download
       by permission.

   Author: 
     Henrik Bærbak Christensen 
     Department of Computer Science
     Aarhus University
   
   Please visit http://www.baerbak.com/ for further information.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at
 
       http://www.apache.org/licenses/LICENSE-2.0
 
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

 
 */
public class BreakthroughSurrogate implements Breakthrough {
  GameState state;

  public BreakthroughSurrogate() {
    state = new GameState();
  }

  @Override
  public Color getPieceAt(Position p) {
    return state.get(p);
  }

  @Override
  public Color getPlayerInTurn() {
    return state.getPlayerInTurn();
  }

  @Override
  public Color getWinner() {
    // As this is a surrogate, winner determination does not
    // make any sense
    return null;
  }

  @Override
  public boolean move(Move m) {
    // The only invalid move is trying to move an opponent's piece (or
    // no piece at all).
    if (state.get(m.getFrom()) != getPlayerInTurn()) return false;

    // Update data structure; note that rules are not observed
    state.doMove(m.getFrom(), m.getTo());

    // Change turns
    state.togglePlayerInTurn();

    return true;
  }

  /** 
   * Get the inner state object for marshalling.
   * @return the state object of the game.
   */
  public GameState getBoardState() {
    return state;
  }
}
